declare const ua: {
    isAndroid(): boolean;
    isDingTalk(): boolean;
    isEdge(): boolean;
    isIE(): boolean;
    isIOS(): boolean;
    isIpad(): boolean;
    isQQ(): boolean;
    isWeixin(): boolean;
    isMobile(): boolean;
    isPC(): boolean;
};

declare type CookieOption = {
    expires?: Date | string | number;
    path?: string;
    domain?: string;
    secure?: boolean | string;
};
declare const Cookie: {
    parse(): Record<string, string>;
    /**
     * get cookie value via key
     * @param key
     */
    get(key?: string): string;
    /**
     * set cookie
     * @param  {string} key
     * @param  {string} value
     * @param  {CookieOption} option
     */
    set(key: string, value: string, option: CookieOption): void;
};

/**
 * string 返回type
 * @param {Any} target
 */
declare const getTypeOf: (target: unknown) => string;

/**
 * safe json parse
 * @param args
 * @returns {[Error | null, any]}
 */
declare const JSONParse: (text: string, reviver?: (this: any, key: string, value: any) => any) => [Error | null, any];

declare type CacheOptions = {
    key?: string | number | symbol | ((...args: any) => any);
    expire?: number;
};
/**
 * async cache
 * @param fn The function needs to be cached, return a promise
 * @param cacheOptions cache options
 * @param cacheOptions.key cache key
 * @param cacheOptions.expire ms number
 * @returns {Function} cached fn
 */
declare const cacheAsync: <T extends (...args: any) => Promise<unknown>>(fn: T, cacheOptions?: CacheOptions) => T;

/**
 * is value can be parsed to a valid Date
 * @param target
 * @returns boolean
 */
declare const isDateValid: (target: Date | string | number | undefined | null) => boolean;

/**
 * is plain object or not
 * @param target
 * @returns boolean
 */
declare const isPlainObject: (target: unknown) => boolean;

/**
 * 校验邮箱地址
 * @param {String} target
 */
declare const isEmail: (target: string) => boolean;

/**
 * 检测是否为数值
 * @param {Any} target
 */
declare const isNumber: (target: string) => boolean;

/**
 * 校验电话号码
 * @param {String} target
 */
declare const isPhone: (target: string) => boolean;

/**
 * 判断是否图片
 * @param {String} target
 */
declare const isImage: (target: string) => boolean;

/**
 * promisify setTimeout
 * @param ms micro second
 * @returns Promise
 */
declare const delay: (ms: number) => Promise<void>;

declare const noop: () => void;

export { Cookie, JSONParse, cacheAsync, delay, getTypeOf, isDateValid, isEmail, isImage, isNumber, isPhone, isPlainObject, noop, ua };
